//----------------------------------------------------------------------------
// C main line
//----------------------------------------------------------------------------

#include <m8c.h>        // part specific constants and macros
#include "PSoCAPI.h"    // PSoC API definitions for all User Modules

/* --------------------------- */
/* --- define/global           */
/* --------------------------- */

#define LCD_CONTRAST_DEFAULT  32


/* --- LCD contrust value (0-100) */
static BYTE g_contrust;
static BYTE g_contrust_comp;

/* --- wait counter (1ms) */
static BYTE g_waitint;

/* --- I2C buffer */
static BYTE g_rcvBuffer[8];
static BYTE rcvCommand[8];
static BYTE g_sendBuffer[8];

/* --- LCD controller mode */
static BYTE g_lcd_mode;

/* --------------------------------------- */
/* --- interrupt                           */
/* --------------------------------------- */
#pragma interrupt_handler Timer1Int
void Timer1Int(void)
{
	if (g_waitint > 0) {
		g_waitint--;
	}
}

void waitASync(BYTE waitval)
{
	g_waitint = waitval;
	while(g_waitint > 0) {}
}


/* --------------------------- */
/* --- LCD controll            */
/* --------------------------- */

void setContrust(BYTE val)
{
	BYTE  light;
	light = val;
	if (light > 254) {
		light = 254;
	}
	DAC8_1_WriteStall(light);
}

void sendCommand(BYTE val)
{
	BYTE i;
	PRT0DR = 0b00000000;	/* RS=0, RW=0 */
	PRT2DR = val;
	PRT0DR = 0b00000001;	/* E=1 */
	for(i=0; i<10; i++) {}
	PRT0DR = 0b00000000;	/* E=0 */
	for(i=0; i<40; i++) {}
}

void sendData(BYTE val)
{
	BYTE i;
	PRT0DR = 0b00001000;	/* RS=0, RW=0 */
	PRT2DR = val;
	PRT0DR = 0b00001001;	/* E=1 */
	for(i=0; i<10; i++) {}
	PRT0DR = 0b00001000;	/* E=0 */	
	for(i=0; i<40; i++) {}
}

void lcdInitialize(void)
{
	waitASync(25);
	/* -- */
	sendCommand(0x30);
	waitASync(5);
	sendCommand(0x30);
	waitASync(1);
	sendCommand(0x30);
	waitASync(5);
	/* -- */
	sendCommand(0x38);
	waitASync(1);
	sendCommand(0x08);
	waitASync(1);
	sendCommand(0x01);
	waitASync(5);
	sendCommand(0x06);
	waitASync(1);
	sendCommand(0x0e);
	waitASync(1);
	/* -- */
	sendCommand(0x3c);
	waitASync(5);
	sendCommand(0x03);
	waitASync(3);
	sendCommand(0x0c);
	waitASync(1);
}

/* --------------------------- */
/* --- I2C work                */
/* --------------------------- */

const BYTE hex[16] = {
  '0','1','2','3','4','5','6','7',
  '8','9','A','B','C','D','E','F'
};

void disphex(BYTE val)
{
	BYTE c, code;
	c = (val & 0xf0) >> 4;
	code = hex[c];
	sendData(code);
	c = val & 0x0f;
	code = hex[c];
	sendData(code);
}

void dispcheck(BYTE *buf)
{
	sendCommand(0x80 | 0x40);
	disphex(buf[0]);
	disphex(buf[1]);
	disphex(buf[2]);
	disphex(buf[3]);
	disphex(buf[4]);
}

void lcd_controller(BYTE *buf)
{
	BYTE  c, sc;
	
	sc = 0;

	if (buf[0] == 0x40) {
		sendData(buf[1]);
	}
	else if (buf[0] == 0) {
		if (buf[1] & 0x80) {
			sendCommand(buf[1]);
		}
		else if (buf[1] == 0x01) {
			sendCommand(buf[1]);
		}
		else if ((buf[1] & 0xfe) == 0x02) {
			sendCommand(buf[1]);
		}
		else if ((buf[1] & 0xc0) == 0x40) {
			if (g_lcd_mode == 0) {
				sendCommand(buf[1]);				
			}
			else {
				if ((buf[1] & 0xf0) == 0x50) {
					g_contrust_comp = (g_contrust_comp & 0x0f) | ((buf[1] & 0x03) << 4);
					sc = 1;
				}
				if ((buf[1] & 0xf0) == 0x70) {
					g_contrust_comp = (g_contrust_comp & 0xf0) | (buf[1] & 0x0f);
					sc = 1;
				}
			}
		}
		else if ((buf[1] & 0xe0) == 0x20) {
			g_lcd_mode = buf[1] & 1;
		}
	}
	
	if (sc != 0) {
		c = 63 - g_contrust_comp;
		c = c + c;
		g_contrust = c;
		setContrust(g_contrust);
	}
}

/* --------------------------- */
/* --- main routine            */
/* --------------------------- */
void main(void)
{
	BYTE status;  

	/* peripheral initialize */
	DAC8_1_Start(DAC8_1_MEDPOWER);
	g_contrust = LCD_CONTRAST_DEFAULT;
	setContrust(g_contrust);
	Counter8_1_EnableInt();
	Counter8_1_Start();
	
	I2CHW_1_Start();
	I2CHW_1_EnableSlave();
	I2CHW_1_InitWrite(g_rcvBuffer, 8);
	I2CHW_1_InitRamRead(g_sendBuffer, 8);
	I2CHW_1_EnableInt();
	
	M8C_EnableGInt;

	lcdInitialize();
	
	sendCommand(0x80);
	sendData('I');
	sendData('2');
	sendData('C');
	sendData(' ');
	sendData('L');
	sendData('C');
	sendData('D');
	
	g_lcd_mode = 0;

	/* --- main loop */
	while(1) {
		status = I2CHW_1_bReadI2CStatus();
		/* --- receive data */
		if ( status & I2CHW_WR_COMPLETE ) {
			rcvCommand[0] = g_rcvBuffer[0];
			rcvCommand[1] = g_rcvBuffer[1];
			I2CHW_1_ClrWrStatus();
			I2CHW_1_InitWrite(g_rcvBuffer, 8);
			/* receive work */
			lcd_controller(rcvCommand);
			//if (rcvCommand[0] != 0x00) dispcheck(rcvCommand);
		}
		if ( status & I2CHW_WR_OVERFLOW ) {
			I2CHW_1_ClrWrStatus();
			I2CHW_1_InitWrite(g_rcvBuffer, 8);		
		}
		/* --- send data */
		if ( status & I2CHW_RD_COMPLETE ) {
			I2CHW_1_ClrRdStatus();  
	        I2CHW_1_InitRamRead(g_sendBuffer, 8);
		}
	}
}
